@@ -4,22 +4,24 @@ use warnings;
 use strict;
 use Carp;
 use Class::InsideOut qw(readonly id register private);
+use File::Copy;
+use File::Temp qw/ tempfile /;
 use JSON;
 use List::Util;
-use version; our $VERSION = qv('1.1.3');
+use version; our $VERSION = qv('1.3.0');
 
 
 use constant FILE_HEADER    => "# config-file-type: JSON 1\n";
 
 
 readonly    getFilePath     => my %filePath;    # path to config file
+readonly    isInclude       => my %isInclude;   # is an include file
 private     config          => my %config;      # in memory config file
+readonly    getIncludes     => my %includes;    # keeps track of any included config files
 
 #-------------------------------------------------------------------
 sub addToArray {
-    my $self = shift;
-    my $property = shift;
-    my $value = shift;
+    my ($self, $property, $value) = @_;
     my $array = $self->get($property);
     unless (defined List::Util::first { $value eq $_ } @{$array}) { # check if it already exists
 		# add it
@@ -28,21 +30,15 @@ sub addToArray {
 	}
 }
 
-
 #-------------------------------------------------------------------
 sub addToHash {
-    my $self = shift;
-    my $property = shift;
-    my $key = shift;
-    my $value = shift;
+    my ($self, $property, $key, $value) = @_;
     $self->set($property."/".$key, $value);
 }
 
-
 #-------------------------------------------------------------------
 sub create {
-	my $class = shift;
-	my $filename = shift;
+	my ($class, $filename) = @_;
     if (open(my $FILE,">",$filename)) {
         print $FILE FILE_HEADER."\n{ }\n";
         close($FILE);
@@ -53,34 +49,34 @@ sub create {
 	return $class->new($filename);	
 }
 
-
-
 #-------------------------------------------------------------------
 sub delete {
-    my $self = shift;
-    my $param = shift;
+    my ($self, $param) = @_;
+	
+	# inform the includes
+	foreach my $include (@{$includes{id $self}}) {
+		$include->delete($param);
+	}
+	
+	# find the directive
     my $directive   = $config{id $self};
     my @parts       = split "/", $param;
     my $lastPart    = pop @parts;
     foreach my $part (@parts) {
         $directive = $directive->{$part};
     }
-    delete $directive->{$lastPart};
-    if (open(my $FILE,">",$self->getFilePath)) {
-        print $FILE FILE_HEADER."\n".JSON->new->pretty->encode($config{id $self});
-        close($FILE);
-    } 
-    else {
-        carp "Can't write to config file ".$self->getFilePath;
-    }
+	
+	# only delete it if it exists
+	if (exists $directive->{$lastPart}) {
+		delete $directive->{$lastPart};
+		$self->write;
+	}
 }
 
 #-------------------------------------------------------------------
 sub deleteFromArray {
-    my $self = shift;
-    my $property = shift;
-    my $value = shift;
-    my $array = $self->get($property);
+    my ($self, $property, $value) = @_;
+    my $array	= $self->get($property);
     for (my $i = 0; $i < scalar(@{$array}); $i++) {
         if ($array->[$i] eq $value) {
             splice(@{$array}, $i, 1);
@@ -90,27 +86,44 @@ sub deleteFromArray {
     $self->set($property, $array);
 }
 
-
 #-------------------------------------------------------------------
 sub deleteFromHash {
-    my $self = shift;
-    my $property = shift;
-    my $key = shift;
+    my ($self, $property, $key) = @_;
     $self->delete($property."/".$key);
 }
 
-
 #-------------------------------------------------------------------
 sub get {
-    my $self        = shift;
-    my $property    = shift;
-    my $value       = $config{id $self};
-    foreach my $part (split "/", $property) {
-        $value = $value->{$part};
-    }
-    return $value;
-}
+    my ($self, $property) = @_;
 
+	# they want a specific property
+	if (defined $property) {
+
+		# look in this config
+		my $value = $config{id $self};
+		foreach my $part (split "/", $property) {
+			$value = $value->{$part};
+		}
+		return $value if (defined $value);
+
+		# look through includes
+		foreach my $include (@{$includes{id $self}}) {
+			my $value = $include->get($property);
+			return $value if (defined $value);
+		}
+
+		# didn't find it
+		return undef;
+	}
+	
+	# they want the whole properties list
+	my %whole = ();
+	foreach my $include (@{$includes{id $self}}) {
+		%whole = (%whole, %{$include->get});			
+	}
+	%whole = (%whole, %{$config{id $self}});
+	return \%whole;
+}
 
 #-------------------------------------------------------------------
 sub getFilename {
@@ -119,23 +132,31 @@ sub getFilename {
     return pop @path;
 }
 
-
 #-------------------------------------------------------------------
 sub new {
-    my $class = shift;
-    my $pathToFile = shift;
+    my ($class, $pathToFile, $options) = @_;
     if (open(my $FILE, "<", $pathToFile)) {
         # slurp
-        my $holdTerminator = $/;
-        undef $/;
+        local $/ = undef;
         my $json = <$FILE>;
-        $/ = $holdTerminator;  
         close($FILE);
-        my $conf = JSON->new->relaxed(1)->decode($json);
+        my $conf;
+        eval {
+            $conf = JSON->new->relaxed(1)->decode($json);
+        };
         croak "Couldn't parse JSON in config file '$pathToFile'\n" unless ref $conf;
-        my $self = register($class);
-        $filePath{id $self} = $pathToFile;
-        $config{id $self}   = $conf;
+        my $self 		= register($class);
+		my $id 			= id $self;
+        $filePath{$id} 	= $pathToFile;
+        $config{$id}   	= $conf;
+        $isInclude{$id} = $options->{isInclude};
+		
+		# process includes
+		my @includes = map { glob $_ } @{ $self->get('includes') || [] };
+		foreach my $include (@includes) {
+			push @{$includes{$id}},  $class->new($include, {isInclude=>1});
+		}
+		
         return $self;
     } 
     else {
@@ -143,31 +164,81 @@ sub new {
     }
 }
 
-
 #-------------------------------------------------------------------
 sub set {
-    my $self        = shift;
-    my $property    = shift;
-    my $value       = shift;
-    my $directive   = $config{id $self};
-    my @parts       = split "/", $property;
-    my $lastPart    = pop @parts;
-    foreach my $part (@parts) {
-        unless (exists $directive->{$part}) {
-            $directive->{$part} = {};
-        }
-        $directive = $directive->{$part};
-    }
-    $directive->{$lastPart} = $value;
-    if (open(my $FILE, ">" ,$self->getFilePath)) {
-        print {$FILE} FILE_HEADER."\n".JSON->new->pretty->encode($config{id $self});
+    my ($self, $property, $value) 	= @_;
+
+	# see if the directive exists in this config
+    my $directive	= $config{id $self};
+    my @parts 		= split "/", $property;
+	my $numParts 	= scalar @parts;
+	for (my $i=0; $i < $numParts; $i++) {
+		my $part = $parts[$i];
+		if (exists $directive->{$part}) { # exists so we continue
+			if ($i == $numParts - 1) { # we're on the last part
+				$directive->{$part} = $value;
+				$self->write;
+				return 1;
+			}
+			else {
+				$directive = $directive->{$part};
+			}
+		}
+		else { # doesn't exist so we quit
+			last;
+		}
+	}
+
+	# see if any of the includes have this directive
+	foreach my $include (@{$includes{id $self}}) {
+		my $found = $include->set($property, $value);
+		return 1 if ($found);
+	}
+
+	# let's create the directive new in this config if it's not an include
+	unless ($self->isInclude) {
+		$directive	= $config{id $self};
+		my $lastPart = pop @parts;
+		foreach my $part (@parts) {
+			unless (exists $directive->{$part}) {
+				$directive->{$part} = {};
+			}
+			$directive = $directive->{$part};
+		}
+	    $directive->{$lastPart} = $value;
+		$self->write;
+		return 1;
+	}
+
+	# didn't find a place to write it	
+	return 0;
+}
+
+#-------------------------------------------------------------------
+sub write {
+	my $self = shift;
+	my $realfile = $self->getFilePath;
+
+	# convert data to json
+    my $json = JSON->new->pretty->encode($config{id $self});
+
+	# create a temporary config file
+	my ($fh, $tempfile) = tempfile();
+	close($fh);
+    if (open(my $FILE,">", $tempfile)) {
+        print $FILE FILE_HEADER."\n".$json;
         close($FILE);
     } 
     else {
-        carp "Can't write to config file ".$self->getFilePath;
+        croak "Can't write (".$realfile.") to temporary file (".$tempfile.")";
     }
+	
+	# move the temp file over the top of the existing file
+	copy($tempfile, $realfile) or croak "Can't copy temporary file (".$tempfile.") to config file (".$realfile.")";
+	unlink $tempfile or carp "Can't delete temporary config file (".$tempfile.")";
 }
 
+
 1; # Magic true value required at end of module
 __END__
 
@@ -178,7 +249,7 @@ Config::JSON - A JSON based config file system.
 
 =head1 VERSION
 
-This document describes Config::JSON version 1.1.3
+This document describes Config::JSON version 1.3.0
 
 
 =head1 SYNOPSIS
@@ -193,11 +264,11 @@ This document describes Config::JSON version 1.1.3
  $config->set($directive,$value);
 
  $config->delete($directive);
- $config->deleteFromHash($name, $key);
- $config->deleteFromArray($name, $value);
+ $config->deleteFromHash($directive, $key);
+ $config->deleteFromArray($directive, $value);
 
- $config->addToHash($name, $key, $value);
- $config->addToArray($name, $value);
+ $config->addToHash($directive, $key, $value);
+ $config->addToArray($directive, $value);
 
  my $path = $config->getFilePath;
  my $filename = $config->getFilename;
@@ -205,7 +276,6 @@ This document describes Config::JSON version 1.1.3
 =head2 Example Config File
 
  # config-file-type: JSON 1
-
  {
         "dsn" : "DBI:mysql:test",
         "user" : "tester",
@@ -218,7 +288,10 @@ This document describes Config::JSON version 1.1.3
         "stats" : {
                 "health" : 32,
                 "vitality" : 11
-        }
+        },
+		
+		# including another file
+		"includes" : ["macros.conf"]
  } 
 
 
@@ -228,10 +301,12 @@ This package parses the config files written in JSON. It also does some non-JSON
 
 If you want to see it in action, it is used as the config file system in WebGUI L<http://www.webgui.org/>.
 
+
 =head2 Why?
 
 Why build yet another config file system? Well there are a number of reasons: We used to use other config file parsers, but we kept running into limitations. We already use JSON in our app, so using JSON to store config files means using less memory because we already have the JSON parser in memory. In addition, with JSON we can have any number of hierarchcal data structures represented in the config file, whereas most config files will give you only one level of hierarchy, if any at all. JSON parses faster than XML and YAML. JSON is easier to read and edit than XML. Many other config file systems allow you to read a config file, but they don't provide any mechanism or utilities to write back to it. JSON is taint safe. JSON is easily parsed by languages other than Perl when we need to do that.
 
+
 =head2 Multi-level Directives
 
 You may of course access a directive called "foo", but since the config is basically a hash you can traverse
@@ -243,10 +318,25 @@ multiple elements of the hash when specifying a directive name by simply delimit
 
 You may do this wherever you specify a directive name.
 
+
 =head2 Comments
 
 You can put comments in the config file as long as # is the first non-space character on the line. However, if you use this API to write to the config file, your comments will be eliminated.
 
+
+=head2 Includes
+
+There is a special directive called "includes", which is an array of include files that may be brought in to
+the config. Even the files you include can have an "includes" directive, so you can do hierarchical includes.
+
+Any directive in the main file will take precedence over the directives in the includes. Likewise the files
+listed first in the "includes" directive will have precedence over the files that come after it. When writing
+to the files, the same precedence is followed.
+
+If you're setting a new directive that doesn't currently exist, it will only be written to the main file.
+
+If a directive is deleted, it will be deleted from all files, including the includes.
+
 =head1 INTERFACE 
 
 =head2 addToArray ( directive, value )
@@ -350,6 +440,12 @@ Returns the filename and path for this config.
 
 
 
+=head2 getIncludes ( )
+
+Returns an array reference of Config::JSON objects that are files included by this config.
+
+
+
 =head2 new ( pathToFile )
 
 Constructor. Builds an object around a config file.
@@ -374,6 +470,10 @@ The value to set the paraemter to. Can be a scalar, hash reference, or array ref
 
 
 
+=head2 write ( )
+
+Writes the file to the filesystem. Normally you'd never need to call this as it's called automatically by the other methods when a change occurs.
+
 
 
 =head1 DIAGNOSTICS
